@@ -590,6 +590,16 @@ fm_icon_view_supports_keep_aligned (FMIconView *view)
 		 supports_keep_aligned, (view));
 }
 
+static gboolean
+fm_icon_view_supports_labels_beside_icons (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), FALSE);
+
+	return EEL_CALL_METHOD_WITH_RETURN_VALUE
+		(FM_ICON_VIEW_CLASS, view,
+		 supports_labels_beside_icons, (view));
+}
+
 static void
 update_layout_menus (FMIconView *view)
 {
@@ -946,6 +956,14 @@ real_supports_keep_aligned (FMIconView *view)
 	return FALSE;
 }
 
+static gboolean
+real_supports_labels_beside_icons (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), TRUE);
+
+	return TRUE;
+}
+
 static gboolean
 set_sort_reversed (FMIconView *icon_view, gboolean new_value)
 {
@@ -1022,6 +1040,26 @@ get_default_zoom_level (void)
 	return CLAMP (default_zoom_level, NAUTILUS_ZOOM_LEVEL_SMALLEST, NAUTILUS_ZOOM_LEVEL_LARGEST);
 }
 
+static void
+set_labels_beside_icons (FMIconView *icon_view)
+{
+	gboolean labels_beside;
+
+	if (fm_icon_view_supports_labels_beside_icons (icon_view)) {
+		labels_beside = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
+		
+		if (labels_beside) {
+			nautilus_icon_container_set_label_position
+				(get_icon_container (icon_view), 
+				 NAUTILUS_ICON_LABEL_POSITION_BESIDE);
+		} else {
+			nautilus_icon_container_set_label_position
+				(get_icon_container (icon_view), 
+				 NAUTILUS_ICON_LABEL_POSITION_UNDER);
+		}
+	}
+}
+
 static void
 fm_icon_view_begin_loading (FMDirectoryView *view)
 {
@@ -1079,6 +1117,8 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 		(get_icon_container (icon_view), 
 		 fm_icon_view_get_directory_tighter_layout (icon_view, file));
 
+	set_labels_beside_icons (icon_view);
+
 	/* We must set auto-layout last, because it invokes the layout_changed 
 	 * callback, which works incorrectly if the other layout criteria are
 	 * not already set up properly (see bug 6500, e.g.)
@@ -1937,6 +1977,44 @@ fm_icon_view_filter_by_screen (FMIconView *icon_view,
 	icon_view->details->num_screens = gdk_display_get_n_screens (gtk_widget_get_display (GTK_WIDGET (icon_view)));
 }
 
+static void
+fm_icon_view_screen_changed (GtkWidget *widget,
+			     GdkScreen *previous_screen)
+{
+	FMDirectoryView *view;
+	GList *files, *l;
+	NautilusFile *file;
+	NautilusIconContainer *icon_container;
+
+	if (GTK_WIDGET_CLASS (parent_class)->screen_changed) {
+		GTK_WIDGET_CLASS (parent_class)->screen_changed (widget, previous_screen);
+	}
+
+	view = FM_DIRECTORY_VIEW (widget);
+	if (FM_ICON_VIEW (view)->details->filter_by_screen) {
+		icon_container = get_icon_container (FM_ICON_VIEW (view));
+		
+		files = nautilus_directory_get_file_list (fm_directory_view_get_model (view));
+
+		for (l = files; l != NULL; l = l->next) {
+			file = l->data;
+			
+			if (!should_show_file_on_screen (view, file)) {
+				fm_icon_view_remove_file (view, file);
+			} else {
+				if (nautilus_icon_container_add (icon_container,
+								 NAUTILUS_ICON_CONTAINER_ICON_DATA (file))) {
+					nautilus_file_ref (file);
+				}
+			}
+		}
+		
+		nautilus_file_list_unref (files);
+		g_list_free (files);
+	}
+}
+
+
 static int
 compare_files_cover (gconstpointer a, gconstpointer b, gpointer callback_data)
 {
@@ -2271,6 +2349,18 @@ default_zoom_level_changed_callback (gpointer callback_data)
 	}
 }
 
+static void
+labels_beside_icons_changed_callback (gpointer callback_data)
+{
+	FMIconView *icon_view;
+
+	g_return_if_fail (FM_IS_ICON_VIEW (callback_data));
+
+	icon_view = FM_ICON_VIEW (callback_data);
+
+	set_labels_beside_icons (icon_view);
+}
+
 static void
 fm_icon_view_sort_directories_first_changed (FMDirectoryView *directory_view)
 {
@@ -2590,6 +2680,8 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 
 	G_OBJECT_CLASS (klass)->finalize = fm_icon_view_finalize;
 	
+	GTK_WIDGET_CLASS (klass)->screen_changed = fm_icon_view_screen_changed;
+	
 	fm_directory_view_class->add_file = fm_icon_view_add_file;
 	fm_directory_view_class->flush_added_files = fm_icon_view_flush_added_files;
 	fm_directory_view_class->begin_loading = fm_icon_view_begin_loading;
@@ -2625,6 +2717,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	klass->clean_up = fm_icon_view_real_clean_up;
 	klass->supports_auto_layout = real_supports_auto_layout;
 	klass->supports_keep_aligned = real_supports_keep_aligned;
+	klass->supports_labels_beside_icons = real_supports_labels_beside_icons;
         klass->get_directory_auto_layout = fm_icon_view_real_get_directory_auto_layout;
         klass->get_directory_sort_by = fm_icon_view_real_get_directory_sort_by;
         klass->get_directory_sort_reversed = fm_icon_view_real_get_directory_sort_reversed;
@@ -2680,6 +2773,9 @@ fm_icon_view_instance_init (FMIconView *icon_view)
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
 						  default_zoom_level_changed_callback,
 						  icon_view, G_OBJECT (icon_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS,
+						  labels_beside_icons_changed_callback,
+						  icon_view, G_OBJECT (icon_view));
 
 	g_signal_connect_object (get_icon_container (icon_view), "handle_uri_list",
 				 G_CALLBACK (icon_view_handle_uri_list), icon_view, 0);
