@@ -191,9 +191,10 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	FMListModel *model;
 	NautilusFile *file;
 	char *str;
-	GdkPixbuf *icon;
+	GdkPixbuf *icon, *tmp;
 	int icon_size;
 	NautilusZoomLevel zoom_level;
+	int width, height;
 
 	model = (FMListModel *)tree_model;
 
@@ -225,6 +226,15 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 		zoom_level = fm_list_model_get_zoom_level_from_column_id (column);
 		icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
 		icon = nautilus_icon_factory_get_pixbuf_for_file (file, NULL, icon_size);
+
+		height = gdk_pixbuf_get_height (icon);
+		if (height > icon_size) {
+			width = gdk_pixbuf_get_width (icon) * icon_size / height;
+			height = icon_size;
+			tmp = gdk_pixbuf_scale_simple (icon, width, height, GDK_INTERP_BILINEAR);
+			g_object_unref (icon);
+			icon = tmp;
+		}
 		g_value_set_object (value, icon);
 		g_object_unref (icon);
 		break;
@@ -554,7 +564,7 @@ each_path_get_data_binder (NautilusDragEachSelectedItemDataGet data_get,
 				uri = nautilus_file_get_uri (file);
 				
 				(*data_get) (uri, 
-					     cell_area.x - info->model->details->drag_begin_x,
+					     0,
 					     cell_area.y - info->model->details->drag_begin_y,
 					     cell_area.width, cell_area.height, 
 					     data);
